//
//  BBSProxy.m
//  rss
//
//  Created by kunii on 09/10/21.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "BBSProxy.h"

/*
	NSMutableDictionary登録用キーワード群。
*/
const NSString* keywords[] = {
	@"",		//	keyword_Name〜keyword_Commentに連動させるため空の文字列を用意した。
	@"name",
	@"date",
	@"comment"
};

@interface BBSProxy (private)	//	libxmlからの呼び出し用メソッド定義
- (void)startElementLocalName:(const xmlChar*)localname 
        prefix:(const xmlChar*)prefix 
        URI:(const xmlChar*)URI 
        nb_namespaces:(int)nb_namespaces 
        namespaces:(const xmlChar**)namespaces 
        nb_attributes:(int)nb_attributes 
        nb_defaulted:(int)nb_defaulted 
        attributes:(const xmlChar**)attributes;
- (void)endElementLocalName:(const xmlChar*)localname 
        prefix:(const xmlChar*)prefix URI:(const xmlChar*)URI;
- (void)charactersFound:(const xmlChar*)ch 
        len:(int)len;
@end

/*
	新しいタグが見つかった。
*/
static void startElementHandler(
        void* ctx, 
        const xmlChar* localname, 
        const xmlChar* prefix, 
        const xmlChar* URI, 
        int nb_namespaces, 
        const xmlChar** namespaces, 
        int nb_attributes, 
        int nb_defaulted, 
        const xmlChar** attributes)
{
	//	BBSProxyのメソッドにそのまま渡す。
    [(BBSProxy*)ctx 
            startElementLocalName:localname 
            prefix:prefix URI:URI 
            nb_namespaces:nb_namespaces 
            namespaces:namespaces 
            nb_attributes:nb_attributes 
            nb_defaulted:nb_defaulted 
            attributes:attributes];
}

/*
	見つかったタグが閉じた。
*/
static void	endElementHandler(
        void* ctx, 
        const xmlChar* localname, 
        const xmlChar* prefix, 
        const xmlChar* URI)
{
	//	BBSProxyのメソッドにそのまま渡す。
    [(BBSProxy*)ctx 
            endElementLocalName:localname 
            prefix:prefix 
            URI:URI];
}

/*
	現在のタグの文字列データが見つかった。
*/
static void	charactersFoundHandler(
        void* ctx, 
        const xmlChar* ch, 
        int len)
{
	//	BBSProxyのメソッドにそのまま渡す。
    [(BBSProxy*)ctx 
            charactersFound:ch len:len];
}

static xmlSAXHandler _saxHandlerStruct = {
    NULL,            /* internalSubset */
    NULL,            /* isStandalone   */
    NULL,            /* hasInternalSubset */
    NULL,            /* hasExternalSubset */
    NULL,            /* resolveEntity */
    NULL,            /* getEntity */
    NULL,            /* entityDecl */
    NULL,            /* notationDecl */
    NULL,            /* attributeDecl */
    NULL,            /* elementDecl */
    NULL,            /* unparsedEntityDecl */
    NULL,            /* setDocumentLocator */
    NULL,            /* startDocument */
    NULL,            /* endDocument */
    NULL,            /* startElement*/
    NULL,            /* endElement */
    NULL,            /* reference */
    charactersFoundHandler, /* characters */
    NULL,            /* ignorableWhitespace */
    NULL,            /* processingInstruction */
    NULL,            /* comment */
    NULL,            /* warning */
    NULL,            /* error */
    NULL,            /* fatalError //: unused error() get all the errors */
    NULL,            /* getParameterEntity */
    NULL,            /* cdataBlock */
    NULL,            /* externalSubset */
    XML_SAX2_MAGIC,  /* initialized */		//	SAXで扱うにはこの値を設定するみたい。
    NULL,            /* private */
    startElementHandler,    /* startElementNs */
    endElementHandler,      /* endElementNs */
    NULL,            /* serror */
};

@implementation BBSProxy
@synthesize bbs;
@synthesize delegate;

-(id)init
{
	if (self = [super init]) {
		bbs = [[NSMutableArray alloc] init];		//	BBSデータ保持用インスタンス作成
		_name = @"iPhoneより";
		_comment = @"";
	}
	return self;
}

-(void)connect
{
	[bbs removeAllObjects];
	// XMLパーサを作成する
	_parserContext = xmlCreatePushParserCtxt(&_saxHandlerStruct, self, NULL, 0, NULL);
	// URLを指定する
	NSMutableURLRequest* request = [[[NSMutableURLRequest alloc] init] autorelease];
	[request setURL:[NSURL URLWithString:@"http://localhost/bbs.php"]];
	[request setHTTPMethod:@"POST"];
	NSString *boundary = [NSString stringWithString:@"xcc2009"];	//	境界文字列
	NSString *contentType = [NSString stringWithFormat:@"multipart/form-data; boundary=%@",boundary];
	[request addValue:contentType forHTTPHeaderField: @"Content-Type"];
	//	内容部作成
	NSMutableData *body = [NSMutableData data];
	//	name=test dataの設定
	[body appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"name\"\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithString:@"Content-Type: text/plain;charset=utf8\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[_name dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	//	location=osaka,japanの設定
	[body appendData:[[NSString stringWithFormat:@"\r\n--%@\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithString:@"Content-Disposition: form-data; name=\"comment\"\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithString:@"Content-Type: text/plain;charset=utf8\r\n\r\n"] dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[_comment dataUsingEncoding:NSUTF8StringEncoding]];
	[body appendData:[[NSString stringWithFormat:@"\r\n--%@--\r\n",boundary] dataUsingEncoding:NSUTF8StringEncoding]];
	//	内容物追加
	[request setHTTPBody:body];
	// URLコネクションを作る
	[NSURLConnection connectionWithRequest:request delegate:self];
}

- (void)connectionDidFinishLoading:(NSURLConnection*)connection
{
	// データが終わった事を伝える。
    xmlParseChunk(_parserContext, NULL, 0, 1);
    // XMLパーサを解放する
	xmlFreeParserCtxt(_parserContext);
	//	通知
	if ([delegate performSelector:@selector(BBSChanged:)])
		[delegate BBSChanged:self];
}

- (void)connection:(NSURLConnection*)connection
        didReceiveData:(NSData*)data
{	
    // データを追加する
    xmlParseChunk(_parserContext, (const char*)[data bytes], [data length], 0);
}

/*
	xmlの一要素にぶつかった時に呼び出される。
	<html>	←1
	<head>	←2
		<title>掲示板</title>	←3
	</head>
*/
- (void)startElementLocalName:(const xmlChar*)localname		//	HTMLのタグが<>を外された形で入っている。例）<form> → form
        prefix:(const xmlChar*)prefix 
        URI:(const xmlChar*)URI 
        nb_namespaces:(int)nb_namespaces 
        namespaces:(const xmlChar**)namespaces 
        nb_attributes:(int)nb_attributes 
        nb_defaulted:(int)nb_defaulted 
        attributes:(const xmlChar**)attributes;
{
	//	divタグにぶつかったなら、新しい項目を用意する。
    if (strncmp((char*)localname, "div", sizeof("div")) == 0) {
        bbsItem = [NSMutableDictionary dictionary];	//	空の辞書作成
        [bbs addObject:bbsItem];
		return;
	}
	//	b, font, blockquoteタグにぶつかったなら新しい文字列バッファを用意する。
	if (strncmp((char*)localname, "b", sizeof("b")) == 0
	|| strncmp((char*)localname, "font", sizeof("font")) == 0
	|| strncmp((char*)localname, "blockquote", sizeof("blockquote")) == 0) {
        // 文字列のためのバッファを作成する
        [curtstring release];
		curtstring = [[NSMutableString string] retain];
	}
}

/*
	startElementLocalNameで見つかったタグが閉じた時に呼び出される。
*/
- (void)endElementLocalName:(const xmlChar*)localname //	HTMLのタグが<>を外された形で入っている。例）</form> → form
        prefix:(const xmlChar*)prefix URI:(const xmlChar*)URI;
{
	//	b, font, blockquoteタグが閉じたなら、それまでに収集した文字列をそのタグのデータ文字列として登録。
	int type = keyword_Unknown;
	if (strncmp((char*)localname, "blockquote", sizeof("blockquote")) == 0)
		type = keyword_Comment;
	else if (strncmp((char*)localname, "font", sizeof("font")) == 0)
		type = keyword_Date;
	else if (strncmp((char*)localname, "b", sizeof("b")) == 0)
		type = keyword_Name;
	if (type != keyword_Unknown) {
        // 文字列を設定する
        [bbsItem setObject:curtstring forKey:keywords[type]];
        [curtstring release];
		curtstring = nil;
    }
}

/*
	現在のタグの文字列データが見つかった。
*/
- (void)charactersFound:(const xmlChar*)ch 
        len:(int)len;
{
    // 文字列を追加する
    if (curtstring) {
        NSString* string = [[NSString alloc] initWithBytes:ch length:len encoding:NSUTF8StringEncoding];
        [curtstring appendString:string];
        [string release];
    }
}

- (void)dealloc {
	[bbs release];
	[_name release];
	[_comment release];
	[super dealloc];
}

/*
	掲示板コメント書き込み
*/
- (void)speak:(NSString*)string 
{
	[_comment release];
	_comment = [string retain];
	[self connect];
}
@end
